// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package impl

import (
	"context"

	"smart-enterprise-management/pb"
	"smart-enterprise-management/pb/v1"

	"github.com/jaryhe/gopkgs/tasker/rpctasker"
	"google.golang.org/grpc"
	"smart-enterprise-management/impl/v1/user"
	"smart-enterprise-management/impl/v1/project"
	"smart-enterprise-management/impl/v1/device"
)

// 具体实现
type Rcvr struct {
}

func Register(s *grpc.Server) {
	pb.RegisterSmartSiteEnterpriseServer(s, &Rcvr{})
}

// 修改密码
func (c *Rcvr) ChangePasswd(ctx context.Context, req *v1.ChangePasswdRequest) (reply *v1.ChangePasswdReply, err error) {
	t1 := func() error {
		reply, err = user.ChangePasswd(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) EmailUpdate(ctx context.Context, req *v1.EmailUpdateRequest) (reply *v1.EmailUpdateReply, err error) {
	t1 := func() error {
		reply, err = user.EmailUpdate(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) EmailPrepare(ctx context.Context, req *v1.EmailPrepareRequest) (reply *v1.EmailPrepareReply, err error) {
	t1 := func() error {
		reply, err = user.EmailPrepare(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) PhoneUpdate(ctx context.Context, req *v1.PhoneUpdateRequest) (reply *v1.PhoneUpdateReply, err error) {
	t1 := func() error {
		reply, err = user.PhoneUpdate(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) UserInfo(ctx context.Context, req *v1.UserInfoRequest) (reply *v1.UserInfoReply, err error) {
	t1 := func() error {
		reply, err = user.UserInfo(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) CompanyInfo(ctx context.Context, req *v1.CompanyInfoRequest) (reply *v1.CompanyInfoReply, err error) {
	t1 := func() error {
		reply, err = user.CompanyInfo(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 注册
func (c *Rcvr) Register(ctx context.Context, req *v1.RegisterRequest) (reply *v1.RegisterReply, err error) {
	t1 := func() error {
		reply, err = user.Register(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) Login(ctx context.Context, req *v1.LoginRequest) (reply *v1.LoginReply, err error) {
	t1 := func() error {
		reply, err = user.Login(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 供应商公示
func (c *Rcvr) Publicity(ctx context.Context, req *v1.PublicityRequest) (reply *v1.PublicityReply, err error) {
	t1 := func() error {
		reply, err = user.Publicity(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 忘记密码设置密码
func (c *Rcvr) SetPasswd(ctx context.Context, req *v1.SetPasswdRequest) (reply *v1.SetPasswdReply, err error) {
	t1 := func() error {
		reply, err = user.SetPasswd(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 添加项目
func (c *Rcvr) ProjectAdd(ctx context.Context, req *v1.ProjectAddRequest) (reply *v1.ProjectAddReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectAdd(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) ProjectInfo(ctx context.Context, req *v1.ProjectInfoRequest) (reply *v1.ProjectInfoReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectInfo(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) ProjectDel(ctx context.Context, req *v1.ProjectDelRequest) (reply *v1.ProjectDelReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectDel(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 项目列表
func (c *Rcvr) ProjectList(ctx context.Context, req *v1.ProjectListRequest) (reply *v1.ProjectListReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 项目账号列表
func (c *Rcvr) ProjectUserList(ctx context.Context, req *v1.ProjectUserListRequest) (reply *v1.ProjectUserListReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectUserList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 启停用项目账号
func (c *Rcvr) EnableProjectUser(ctx context.Context, req *v1.EnableProjectUserRequest) (reply *v1.EnableProjectUserReply, err error) {
	t1 := func() error {
		reply, err = project.EnableProjectUser(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 企业审批状态
func (c *Rcvr) CompanyApproveStatus(ctx context.Context, req *v1.CompanyApproveStatusRequest) (reply *v1.CompanyApproveStatusReply, err error) {
	t1 := func() error {
		reply, err = user.CompanyApproveStatus(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 设备类型列表
func (c *Rcvr) DeviceTypeList(ctx context.Context, req *v1.DeviceTypeListRequest) (reply *v1.DeviceTypeListReply, err error) {
	t1 := func() error {
		reply, err = device.DeviceTypeList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}


func (c *Rcvr) DeviceAllTypeList(ctx context.Context, req *v1.DeviceAllTypeListRequest) (reply *v1.DeviceAllTypeListReply, err error) {
	t1 := func() error {
		reply, err = device.DeviceAllTypeList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 设备列表
func (c *Rcvr) DeviceList(ctx context.Context, req *v1.DeviceListRequest) (reply *v1.DeviceListReply, err error) {
	t1 := func() error {
		reply, err = device.DeviceList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 视屏设备下的通道列表
func (c *Rcvr) ChannelList(ctx context.Context, req *v1.ChannelListRequest) (reply *v1.ChannelListReply, err error) {
	t1 := func() error {
		reply, err = device.ChannelList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 视屏设备列表
func (c *Rcvr) VedioList(ctx context.Context, req *v1.VedioListRequest) (reply *v1.VedioListReply, err error) {
	t1 := func() error {
		reply, err = device.VedioList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

// 重置项目账号密码
func (c *Rcvr) ProjectUserPasswordReset(ctx context.Context, req *v1.ProjectUserPasswordResetRequest) (reply *v1.ProjectUserPasswordResetReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectUserPasswordReset(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}


func (c *Rcvr) ProjectUpdate(ctx context.Context, req *v1.ProjectUpdateRequest) (reply *v1.ProjectUpdateReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectUpdate(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}


func (c *Rcvr) ProjectDistrict(ctx context.Context, req *v1.ProjectDistrictRequest) (reply *v1.ProjectDistrictReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectDistrict(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}



func (c *Rcvr) ProjectFinish(ctx context.Context, req *v1.ProjectFinishRequest) (reply *v1.ProjectFinishReply, err error) {
	t1 := func() error {
		reply, err = project.ProjectFinish(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}

func (c *Rcvr) AreaCodeList(ctx context.Context, req *v1.AreaCodeListRequest) (reply *v1.AreaCodeListReply, err error) {
	t1 := func() error {
		reply, err = project.AreaCodeList(ctx, req)
		return err
	}

	return reply, rpctasker.Exec(ctx, t1)
}