// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package device

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-enterprise-management/consts"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)
var layout = "2006-01-02 15:04:05"
func DeviceTypeList(ctx context.Context, req *v1.DeviceTypeListRequest) (reply *v1.DeviceTypeListReply, err error) {
	reply = &v1.DeviceTypeListReply{}
	p := &model.TDeviceTypeItem{}
	m, _, err := p.AllMap(database.DB())
	if err !=  nil {
		logger.Error("DeviceTypeList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if req.Page < 1 {
		req.Page = 1
	}
	pdt := &model.ProviderDeviceTesting{}
	list, total, err := pdt.ListWithProvider(database.DB(), req.Page, req.Filter, req.FilterStatus)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("DeviceTypeList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.Total = total
	reply.Page = req.Page
	reply.PageSize = int32(model.PageSize)
	if total == 0 {
		return reply, nil
	}
	reply.List = make([]*v1.DeviceTypeItem, len(list))
	for i, v := range list {
		reply.List[i] = &v1.DeviceTypeItem{}
		reply.List[i].Id = v.Id
		reply.List[i].Status = v.Status
		reply.List[i].SocialCode = v.SocialCode
		reply.List[i].ApplyTime = v.CreatedAt.Format(layout)
		reply.List[i].Comment = v.Comment
		reply.List[i].ProviderName = v.ProviderName
		reply.List[i].TypeCode = v.TypeCode
		reply.List[i].TypeName = m[v.TypeCode]
		reply.List[i].Feedback = v.Feedback
		if v.Status == consts.DeviceTypeStatusAddTestted || v.Status == consts.DeviceTypeStatusAddTesttedFail {
			reply.List[i].DevelopApproveTime = v.DevelopApproveTime.Format(layout)
		}

	}

	return reply, nil
}
