// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package device

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"smart-enterprise-management/consts"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

func VedioList(ctx context.Context, req *v1.VedioListRequest) (reply *v1.VedioListReply, err error) {
	reply = &v1.VedioListReply{}
	if req.Cid == 0 {
		return nil, errors.ParamsError
	}

	if req.Page < 1 {
		req.Page = 1
	}

	pt := &model.TDeviceTypeItem{}
	m, _, err := pt.AllMap(database.DB())
	if err != nil {
		logger.Error("DeviceList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	dbReq := model.VedioDeviceListRequest{
		Page:req.Page,
		ProjectId:req.ProjectId,
		ProviderId:req.ProviderId,
		Filter:req.Filter,
		StatusFilter:req.StatusFilters,
		IsAll:req.IsAll,
		Cid:req.Cid,
		State:req.State,
	}
	list, total, err := model.VedioDeviceList(database.DB(), dbReq)
	if err != nil {
		logger.Error("VedioList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.Page = req.Page
	reply.Total = total
	reply.PageSize = int32(model.PageSize)
	if total == 0 {
		return reply, nil
	}

	reply.List = make([]*v1.VedioItem, len(list))
	for i, v := range list{
		item := &v1.VedioItem{
			Id:             int64(v.Id),
			Sn:             v.Sn,
			TypeCode:       v.TypeCode,
			TypeName:       m[v.TypeCode],
			ProjectName:    v.ProjectName,
			SafetyRecordNo: v.SafetyRecordNo,
			ApplyTime:    v.CreatedTime.Format(layout),
			Status:v.Status,
			State:v.State,
			Key:v.Key,
			SocialCode:v.SocialCode,
			ProviderName:v.ProviderName,
			Name:v.Name,
			Ip:v.Ip,
			Port:int64(v.Port),
			ChannelCount:int32(v.ChannelCount),
			MediaTransport:v.MediaTransport,

		}
		if item.Status == consts.DeviceStatusAddAuditted ||
			item.Status == consts.DeviceStatusAddAudittedFail {
			item.ApproveTime = v.ProjectApproveTime.Format(layout)
		}

		reply.List[i] = item
	}
	reply.Page = req.Page
	reply.Total = total
	reply.PageSize = int32(model.PageSize)

	return reply, nil
}
