// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"encoding/json"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"github.com/jaryhe/gopkgs/tasker/transtasker"
)


func getDeviceCount(projectId int64) (int64, error) {
	where := map[string]interface{}{
		"ProjectId":projectId,
		"VerifyStatus":1,
	}
	p := model.DeviceAll{}
	count, err := p.Count(database.DB(), where)
	if err != nil {
		return 0, errors.DataBaseError
	}
	return count, nil
}

func ProjectDel(ctx context.Context, req *v1.ProjectDelRequest) (reply *v1.ProjectDelReply, err error) {
	if req.Id < 1 {
		return nil, errors.ParamsError
	}
	count, err := getDeviceCount(req.Id)
	if err != nil {
		return nil, err
	}
	if count > 0 {
		return nil, errors.ProjectDelError
	}
	origin := ""
	delTask := func(db *gorm.DB) error {
		where := map[string]interface{}{
			"ID":req.Id,
		}
		// 获取项目原始数据
		project := model.ProjectInfo{}
		err := project.Find(db, where)
		if err != nil && err != gorm.ErrRecordNotFound {
			logger.Error("ProjectDel project find",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if project.ID == 0 {
			return errors.NoRecordError
		}
		bytes, _ := json.Marshal(project)
		origin = string(bytes)

		// 删除项目
		project.ID = 0
		err = project.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel project",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		// 删除视屏
		err = model.DelVedio(db, req.Id)
		if err != nil {
			logger.Error("ProjectDel vedio",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		where = map[string]interface{}{
			"ProjectId":req.Id,
		}

		// 删除设备
		device := model.DeviceAll{}
		err = device.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel device",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除项目账号
		user := model.TSiteUser{}
		err = user.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel user",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除工程进度
		progress := model.TProjectSchedule{}
		err = progress.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel schedule",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除告警规则
		alarmRule := model.TAlarmRule{}
		err = alarmRule.Delete(db, where)
		if err != nil {
			logger.Error("ProjectDel alarm rule",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除告警联系人
		alarmContact := model.TAlarmContact{}
		err = alarmContact.Delete(db, where)
		if err != nil {
			logger.Error("ProjectDel alarm contact",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除告警数据
		alarm := model.TAlarm{}
		err = alarm.Delete(db, where)
		if err != nil {
			logger.Error("ProjectDel alarm",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		// 删除工单
		pjob := model.DeviceDelJob{}
		err = pjob.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel job",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除扬尘设备
		dust := model.DustDeviceInfo{}
		err = dust.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel job",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除运渣车
		slag := model.SlagcarDevice{}
		err = slag.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel job",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
/*
		// 删除扬尘数据
		dustData := model.DustMonitorInfo{}
		err = dustData.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel job",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除扬尘超标数据
		dustExcessive := model.DustExcessiveInfo{}
		err = dustExcessive.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel job",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除扬尘超标图片
		dustExcessiveImg := model.DustExcessiveImg{}
		err = dustExcessiveImg.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel job",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

 */
		// 删除管理人员信息
		pm := model.ProjectPMInfo{}
		err = pm.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel job",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 删除参建单位信息
		corp := model.ProjectCorpInfo{}
		err = corp.Del(db, where)
		if err != nil {
			logger.Error("ProjectDel job",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		return nil
	}
	reply = &v1.ProjectDelReply{}
	err = transtasker.Exec(database.DB(),
		transtasker.NewTask(delTask))
	reply.Origin = origin
	return reply, err
}
