// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"time"
)

func ProjectFinish(ctx context.Context, req *v1.ProjectFinishRequest) (reply *v1.ProjectFinishReply, err error) {
	if req.Id < 1 {
		return nil, errors.ParamsError
	}
	now := time.Now()
	reply = &v1.ProjectFinishReply{}

	p := model.ProjectInfo{}
	and := map[string]interface{}{
		"ID": req.Id,
	}
	err = p.FindOr(database.DB(), and, nil)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectFinish",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ProjectNotExistError
	}
	progress := model.TProjectSchedule{}
	where := map[string]interface{}{
		"ProjectId": req.Id,
		"Status !=":  1,
	}
	count, err := progress.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if count > 0 {
		return nil, errors.ProjectFinishError
	}

	p = model.ProjectInfo{}
	and = map[string]interface{}{
		"ID": req.Id,
	}
	values := map[string]interface{}{
		"PrjStatus":  4,
		"UpdatedAt": now,
	}
	err = p.Update(database.DB(), and, values)
	if err != nil {
		logger.Error("ProjectFinish",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	return reply, nil

}
