// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"encoding/base64"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-enterprise-management/consts"
	"strings"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"github.com/jaryhe/gopkgs/util"
)


func certDecrypt(cert string) (string) {
	bytes,_ := base64.StdEncoding.DecodeString(cert)
	ret, _ := util.AesDecrypt(bytes, []byte("2D29BP43U3Y1B4N6REQW3F319DD23454"))
	return string(ret)
}
func ProjectInfo(ctx context.Context, req *v1.ProjectInfoRequest) (reply *v1.ProjectInfoReply, err error) {
	if req.Id < 1 {
		return nil, errors.ParamsError
	}
	where := map[string]interface{}{
		"ID": req.Id,
	}
	p := model.ProjectInfo{}
	p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.NoRecordError
	}
	reply = &v1.ProjectInfoReply{}
	reply.Info = &v1.ProjectInfo{}
	reply.Info.Id = p.ID
	reply.Info.Location = p.Address
	reply.Info.Name = p.Name
	reply.Info.SafetyRecordNo = p.Safetyno
	reply.Info.Lon = p.Lng
	reply.Info.Lat = p.Lat
	reply.Info.Price = p.Invest
	reply.Info.CoveredArea = p.Buildingarea
	reply.Info.Desc = p.Description
	reply.Info.StartDay = p.Startdate.Format("2006-01-02")
	reply.Info.EndDay = p.Enddate.Format("2006-01-02")
	reply.Info.CreateTime = p.Createdat.Format("2006-01-02 15:04:05")
	reply.Info.ProjectStatus = p.Prjstatus
	reply.Info.Category = p.Category
	reply.Info.ConstructType = p.Constructtype
	reply.Info.InvestType = p.InvestType
	reply.Info.Scale = p.Scale
	reply.Info.ProvincialVerifyStatus = consts.ProvincialVerifyStatus[p.ProvincialVerifyStatus]
	reply.Info.ProvincialVerifyResult = consts.ProvincialVerifyStatus[p.ProvincialVerifyCode]
	reply.Info.ProvincialVerifyInfo = p.ProvincialVerifyInfo

	if !strings.Contains(p.Planpic, "[") {
		reply.Info.ProjectPlan = "[]"
	} else {
		reply.Info.ProjectPlan = p.Planpic
	}
	if !strings.Contains(p.Effectpic, "[") {
		reply.Info.ProjectEffectPic = "[]"
	} else {
		reply.Info.ProjectEffectPic = p.Effectpic
	}

	corp := model.ProjectCorpInfo{}
	where = map[string]interface{}{
		"ProjectId":req.Id,
	}
	corpList, err := corp.All(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.CorpList = make([]*v1.ProjectCorpListItem, len(corpList))
	for i, v := range corpList {
		item := &v1.ProjectCorpListItem{
			CorpType:consts.ProjectCorpM[v.CorpType],
			Name:v.CorpName,
			AreaCode:v.AreaCode,
			Id:v.ID,
			RegisterDate:v.RegisterDate.Format(layout),
			SocialCode:v.CorpCode,
		}
		reply.CorpList[i] = item
	}



	pm := model.ProjectPMInfo{}

	pmList, err := pm.All(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.PmList = make([]*v1.ProjectPMListItem, len(pmList))
	for i, v := range pmList {
		if reply.Info.Manager == "" && v.PType == consts.StaffTypeManager {
			reply.Info.Manager = v.PMName
			reply.Info.ManagerPhone = v.PMPhone
		}
		plainCert := certDecrypt(v.PMIDCardNumber)
		item := &v1.ProjectPMListItem{
			CorpType:consts.ProjectCorpM[v.CorpType],
			CorpName:v.CorpName,
			Id:v.ID,
			SocialCode:v.CorpCode,
			CertType:consts.CertTypeM[v.PMIDCardType],
			PType:consts.StaffTypeM[v.PType],
			Name:v.PMName,
			Phone:v.PMPhone,
			CertNum:plainCert,
		}
		reply.PmList[i] = item
	}


	return reply, nil
}
