// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-enterprise-management/consts"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"strings"
)

var layout = "2006-01-02 15:04:05"
func ProjectList(ctx context.Context, req *v1.ProjectListRequest) (reply *v1.ProjectListReply, err error) {

	if req.Page < 1 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	if req.Cid < 1 {
		return nil, errors.ParamsError
	}
	p := model.ProjectInfo{}
	list, total, err := p.ListWithPm(database.DB(), req.Filter, req.Cid, req.Page, req.PageSize)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	reply = &v1.ProjectListReply{
		Total:total,
		PageSize:int32(model.PageSize),
		Page:req.Page,
	}
	if total == 0 {
		return reply, nil
	}

	reply.List = make([]*v1.ProjectItem, len(list))
	for i, v := range list {
		reply.List[i] = &v1.ProjectItem{}

		reply.List[i].Id = v.ID
		reply.List[i].Location = v.Address
		reply.List[i].Name = v.Name
		reply.List[i].SafetyRecordNo = v.Safetyno
		reply.List[i].Lon = v.Lng
		reply.List[i].Lat = v.Lat
		reply.List[i].Price = v.Invest
		reply.List[i].CoveredArea = v.Buildingarea
		reply.List[i].Desc = v.Description
		reply.List[i].StartDay = v.Startdate.Format("2006-01-02")
		reply.List[i].EndDay = v.Enddate.Format("2006-01-02")
		reply.List[i].CreateTime = v.Createdat.Format("2006-01-02 15:04:05")
		reply.List[i].ProjectStatus = consts.ProjectStatusM[v.Prjstatus]
		reply.List[i].Category = consts.ProjectCategoryM[v.Category]
		reply.List[i].ConstructType = consts.ProjectConstructTypeM[v.Constructtype]

		if !strings.Contains(v.Planpic, "[") {
			reply.List[i].ProjectPlan = "[]"
		} else {
			reply.List[i].ProjectPlan = v.Planpic
		}
		if !strings.Contains(v.Effectpic, "[") {
			reply.List[i].ProjectEffectPic = "[]"
		} else {
			reply.List[i].ProjectEffectPic = v.Effectpic
		}
		reply.List[i].ProvincialVerifyStatus = consts.ProvincialVerifyStatus[v.ProvincialVerifyStatus]
		reply.List[i].ProvincialVerifyResult = consts.ProvincialVerifyStatus[v.ProvincialVerifyCode]
		reply.List[i].ProvincialVerifyInfo = v.ProvincialVerifyInfo
		reply.List[i].Manager = v.Manager
		reply.List[i].ManagerPhone = v.ManagerPhone
	}

	return reply, nil
}
