// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

func EnableProjectUser(ctx context.Context, req *v1.EnableProjectUserRequest) (reply *v1.EnableProjectUserReply, err error) {
	if req.Id < 1 {
		return nil, errors.ParamsError
	}
	p := &model.TSiteUser{}
	where := map[string]interface{}{
		"ID":req.Id,
	}
	err = p.Find(database.DB(), where, nil)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.Id == 0 {
		return nil, errors.NoRecordError
	}
	projectId := p.ProjectId
	value := map[string]interface{}{
		"Enable":req.Enable,
	}
	err = p.Update(database.DB(), where, value)
	if err != nil {
		logger.Error("EnableProjectUser",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply = &v1.EnableProjectUserReply{ProjectId:projectId}

	return reply, nil
}
