// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"encoding/base64"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"github.com/jaryhe/gopkgs/util"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

const CRYPTO_KEY = "3D28BP43U3Y1B4N6REQW3F319DD23467"

func getOriginPass(passwd string) (string) {
	bytes,_ := base64.StdEncoding.DecodeString(passwd)
	ret, _ := util.AesDecrypt(bytes, []byte(CRYPTO_KEY))
	return string(ret)
}

func ProjectUserList(ctx context.Context, req *v1.ProjectUserListRequest) (reply *v1.ProjectUserListReply, err error) {
	if req.Cid < 1 {
		return nil, errors.ParamsError
	}
	p := &model.TSiteUser{}
	reply = &v1.ProjectUserListReply{}
	list, total, err := p.ListWithProject(database.DB(), req.Filter, req.Cid, req.Page)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectUserList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.Total = total
	reply.PageSize = int32(model.PageSize)
	reply.Page = req.Page
	if total == 0 {
		return reply, nil
	}
	reply.List = make([]*v1.ProjectUserItem, len(list))
	for i, v := range list {
		reply.List[i] = &v1.ProjectUserItem{}
		reply.List[i].Id = v.Id
		reply.List[i].Enable = v.Enable
		reply.List[i].Username = v.Username
		reply.List[i].ProjectName = v.ProjectName
		reply.List[i].SafetyRecordNo = v.SafetyRecordNo
		reply.List[i].Passwd = getOriginPass(v.Passwd)
		reply.List[i].UserType = v.UserType
	}

	return reply, nil
}
