package user

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"strings"
)
var layout = "2006-01-02 15:04:05"
func CompanyInfo(ctx context.Context, req *v1.CompanyInfoRequest) (reply *v1.CompanyInfoReply, err error) {
	p := model.TCompany{}
	if req.Id == 0 {
		return nil, errors.ParamsError
	}

	err = p.Find(database.DB(), map[string]interface{}{"ID": req.Id}, nil)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("CompanyList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	reply = &v1.CompanyInfoReply{}
	reply.Info = &v1.CompanyItem{}
	reply.Info.Status = int32(p.Status)
	reply.Info.ApplyTime = p.CreatedAt.Format(layout)
	if p.Status > 0 {
		reply.Info.ApproveTime = p.ApproveTime.Format(layout)
	}
	reply.Info.Name = p.Name
	reply.Info.SocialCode = p.SocialCode
	reply.Info.Id = p.Id
	reply.Info.BusinessContactPhone = p.BusinessContactPhone
	reply.Info.BusinessContact = p.BusinessContact
	reply.Info.Province = p.Province
	reply.Info.City = p.City
	reply.Info.Zone = p.Zone
	reply.Info.Feedback = p.Feedback
	reply.Info.LegalPerson = p.LegalPerson
	reply.Info.UserName = p.UserName
	reply.Info.BusinessResponsible = p.BusinessResponsible
	reply.Info.BusinessResponsiblePhone = p.BusinessResponsiblePhone
	reply.Info.IdCert = p.IdCert
	reply.Info.EmergencyContact = p.EmergencyContact
	reply.Info.EmergencyContactPhone = p.EmergencyContactPhone
	reply.Info.BusinessLicense = p.BusinessLicense
	reply.Info.Reviewer = p.Reviewer
	if !strings.Contains(p.BusinessLicense, "[") {
		reply.Info.BusinessLicense = "[]"
	}

	return reply, nil
}
