package user

import (
	"context"
	"encoding/base64"
	"encoding/json"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/tasker/transtasker"
	"github.com/jaryhe/gopkgs/util"
	"github.com/jinzhu/gorm"
	"smart-enterprise-management/consts"
	"smart-enterprise-management/errors"
	dbmodel "smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"smart-enterprise-management/utils"
	"time"
	"fmt"
	"google.golang.org/grpc/status"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"smart-enterprise-management/pb"
)

const CRYPTO_KEY = "3D28BP43U3Y1B4N6REQW3F319DD23467"

func getB64Pass(passwd string) (string, error) {
	cryPasswd, err := util.AesEncrypt(passwd, CRYPTO_KEY)
	if err != nil {
		return "", errors.ParamsError
	}
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret, nil
}

func getMd5Pass(passwd string) (string, error) {
	cryPasswd, err := util.AesEncrypt(passwd, CRYPTO_KEY)
	if err != nil {
		return "", errors.ParamsError
	}
	ret := util.MD5(base64.StdEncoding.EncodeToString(cryPasswd))
	return ret, nil
}

func getOriginPass(passwd string) (string) {
	bytes,_ := base64.StdEncoding.DecodeString(passwd)
	ret, _ := util.AesDecrypt(bytes, []byte(CRYPTO_KEY))
	return string(ret)
}

func Passtest() {
	b64, _ := getB64Pass("123456")
	fmt.Printf("%v\n", b64)

	origin := getOriginPass(b64)
	fmt.Printf("origin:%v\n", origin)
}

func checkRegisterParam(req *v1.RegisterRequest) error {
	if req.BusinessContact == "" ||
		req.BusinessContactPhone == "" ||
		req.Name == "" ||
		req.SocialCode == "" ||
		req.Passwd == "" ||
		req.UserName == "" ||
		req.IdCert == "" ||
		len(req.BusinessLicense) == 0 ||
		req.BusinessResponsible == "" ||
		req.BusinessResponsiblePhone == "" ||
		req.EmergencyContactPhone == ""{
		return errors.ParamsError
	}
	if req.Vcode == 0 {
		return status.Error(10003, "验证码为空")
	}
	if !utils.VerifyMobileFormat(req.BusinessResponsiblePhone) {
		return status.Error(10003, "业务负责人手机号格式错误")
	}
	if !utils.VerifyMobileFormat(req.BusinessContactPhone) {
		return status.Error(10003, "业务联系人手机号格式错误")
	}
	if !utils.VerifyMobileFormat(req.EmergencyContactPhone) {
		return status.Error(10003, "紧急联系人手机号格式错误")
	}
	if _, ok := utils.CheckIDCert(req.IdCert); !ok {
		return status.Error(10003, "身份证格式错误")
	}

	rpcReq := &v1.CheckVcodeRequest{
		PhoneNumber:req.BusinessContactPhone,
		Vcode:req.Vcode,
	}

	_, err := pb.Thirdparty.CheckVcode(context.Background(), rpcReq)

	if err != nil {
		s, _ := json.Marshal(rpcReq)
		logger.Error("func",
			zap.String("call", "pb.Thirdparty.CheckVcode"),
			zap.String("params", string(s)),
			zap.String("error", err.Error()))

		return err
	}
	return nil
}

func Register(ctx context.Context, req *v1.RegisterRequest) (reply *v1.RegisterReply, err error) {
	if err := checkRegisterParam(req); err != nil {
		return nil, err
	}
	step1 := func(db *gorm.DB) error {

		p := &dbmodel.TCompany{}

		and := map[string]interface{}{
			"Status !=":consts.CompanyStatusAddAudittedFail,
		}
		or := map[string]interface{}{
			"SocialCode":req.SocialCode,
			"UserName":req.UserName,
			"BusinessContactPhone":req.BusinessContactPhone,
		}
		// 信用码是否已存在
		err = p.Find(database.DB(), and, or)
		if err != nil && err != gorm.ErrRecordNotFound {
			logger.Error("Register",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.Id > 0 {
			return errors.UserExistError
		}

		return nil
	}

	step2 := func(db *gorm.DB) error {
		p := &dbmodel.TCompany{}
		utils.StructCopy(p, req, "")
		if len(req.BusinessLicense) > 0 {
			bytes, _ := json.Marshal(req.BusinessLicense)
			p.BusinessLicense = string(bytes)
		}

		p.Passwd, err = getMd5Pass(req.Passwd)
		if err != nil {
			logger.Error("Register",
				zap.String("err", err.Error()))
			return errors.SystemError
		}
		now := time.Now()
		p.CreatedAt = now
		p.UpdatedAt = now

		err = p.Insert(database.DB())
		if err != nil {
			logger.Error("Register",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		return nil
	}

	err = transtasker.Exec(database.DB(),
		transtasker.NewTask(step1),
		transtasker.NewTask(step2))
	return &v1.RegisterReply{},err
}
