// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package pb

import (
	"fmt"
	"time"
	"smart-enterprise-management/parser"

	"google.golang.org/grpc"
	"google.golang.org/grpc/keepalive"
)

// 客户端集合
var Thirdparty SmartSiteThirdpartyClient
var Provincial SmartProvincialClient


func setupSmartSiteClient(kacp keepalive.ClientParameters, conns []*grpc.ClientConn) {
	// 根据是否为k8s来组装targets
	var serviceName string

	if parser.Conf.K8s {
		serviceName = parser.Conf.Rpc.SmartThirdparty.ServiceName
	} else {
		serviceName = parser.Conf.Rpc.SmartThirdparty.ServiceIp
	}

	// 发起一个连接并记录连接conn，后期释放
	if conn, err := grpc.Dial(fmt.Sprintf("%s:%d", serviceName,
		parser.Conf.Rpc.SmartThirdparty.ServicePort),
		grpc.WithInsecure(), grpc.WithKeepaliveParams(kacp)); err == nil {
		Thirdparty = NewSmartSiteThirdpartyClient(conn)
		conns = append(conns, conn)
	} else {
		fmt.Println("[rpc] dial thirdparty conn err", err)
	}


	return
}

func setupSmartProvincialClient(kacp keepalive.ClientParameters, conns []*grpc.ClientConn) {
	// 根据是否为k8s来组装targets
	var serviceName string

	if parser.Conf.K8s {
		serviceName = parser.Conf.Rpc.SmartProvincial.ServiceName
	} else {
		serviceName = parser.Conf.Rpc.SmartProvincial.ServiceIp
	}

	// 发起一个连接并记录连接conn，后期释放
	if conn, err := grpc.Dial(fmt.Sprintf("%s:%d", serviceName,
		parser.Conf.Rpc.SmartProvincial.ServicePort),
		grpc.WithInsecure(), grpc.WithKeepaliveParams(kacp)); err == nil {
		Provincial = NewSmartProvincialClient(conn)
		conns = append(conns, conn)
	} else {
		fmt.Println("[rpc] dial provincial conn err", err)
	}


	return
}


// SetupClients 创建客户端
func SetupClients() (conns []*grpc.ClientConn) {
	// 客户端配置参数
	var kacp = keepalive.ClientParameters{
		// send pings every n seconds if there is no activity
		Time: time.Duration(parser.Conf.Rpc.Keepalive.ClientTime) * time.Second,
		// wait n second for ping ack before considering the connection dead
		Timeout: time.Duration(parser.Conf.Rpc.Keepalive.ClientTimeout) * time.Second,
		// send pings even without active streams
		PermitWithoutStream: true,
	}

	setupSmartSiteClient(kacp, conns)
	setupSmartProvincialClient(kacp, conns)

	return
}
