// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package utils

import (
	"regexp"
	"strconv"
	"strings"
	"time"
	"math/rand"
	"fmt"
)

var (
	idCertMatrix       = []int{7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2}
	idCertVerifyCode   = []byte("10X98765432")
	socialCreditMatrix = []int{1, 3, 9, 27, 19, 26, 16, 17, 20, 29, 25, 13, 8, 24, 10, 30, 28}
	socialCreditMap    = map[int]int{'0': 0, '1': 1, '2': 2, '3': 3, '4': 4, '5': 5, '6': 6, '7': 7, '8': 8, '9': 9,
		'A': 10, 'B': 11, 'C': 12, 'D': 13, 'E': 14, 'F': 15, 'G': 16, 'H': 17, 'J': 18,
		'K': 19, 'L': 20, 'M': 21, 'N': 22, 'P': 23, 'Q': 24, 'R': 25, 'T': 26, 'U': 27,
		'W': 28, 'X': 29, 'Y': 30}

	socialCreditMapKeys   = []int{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'T', 'U', 'W', 'X', 'Y'}
	socialCreditMapValues = []int{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30}
)

const (
	NUmStr  = "0123456789"
	CharStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
	SpecStr = "+=-@#~,.[]()!%^*$"
)


func CheckIDCert(str string) (realIdCert string, res bool) {

	defer func() {
		if r := recover(); r != nil {
			res = false
		}
	}()

	idCert := strings.ToUpper(str)
	idCert = strings.TrimSpace(idCert)
	ret := idCert
	if len(idCert) != 18 {
		return "", false
	}

	year, err := strconv.Atoi(idCert[6:10])

	if err != nil {
		return "", false
	}

	if !(1900 < year && year < 2100) {
		return "", false
	}

	check := 0
	lastLetter := 0
	for index, value := range []byte(idCert) {
		if index == 17 {
			lastLetter = int(value)
			break
		}

		if !(value >= 48 && value <= 57) {
			return "", false
		}

		v := value - 48
		check = check + idCertMatrix[index]*int(v)
	}

	if !((lastLetter >= 48 && lastLetter <= 57) || lastLetter == 'X') {
		return "", false
	}

	timeLayout := "20060102"                                     //转化所需模板
	loc, _ := time.LoadLocation("Local")                         //重要：获取时区
	_, err = time.ParseInLocation(timeLayout, idCert[6:14], loc) //使用模板在对应时区转化为time.time类型

	if err != nil {
		return "", false
	}

	verifyCode := int(idCertVerifyCode[check%11])
	if lastLetter == verifyCode {
		return ret, true
	}
	return "", false
}

func VerifyMobileFormat(phone string) bool {
	regular := "^((13[0-9])|(14[5,7])|(15[0-3,5-9])|(17[0,3,5-8])|(18[0-9])|166|198|199|(147))\\d{8}$"

	reg := regexp.MustCompile(regular)
	return reg.MatchString(phone)
}

func GenerateRandomStr(length int, charset string) string {
	time.Sleep(1*time.Microsecond)
	rand.Seed(time.Now().UnixNano())
	//初始化密码切片
	var passwd []byte = make([]byte, length, length)
	//源字符串
	var sourceStr string
	//判断字符类型,如果是数字
	if charset == "num" {
		sourceStr = NUmStr
		//如果选的是字符
	} else if charset == "char" {
		sourceStr = charset
		//如果选的是混合模式
	} else if charset == "mix" {
		sourceStr = fmt.Sprintf("%s%s", NUmStr, CharStr)
		//如果选的是高级模式
	} else if charset == "advance" {
		sourceStr = fmt.Sprintf("%s%s%s", NUmStr, CharStr, SpecStr)
	} else {
		sourceStr = fmt.Sprintf("%s%s%s", NUmStr, CharStr, SpecStr)
	}
	//遍历，生成一个随机index索引,
	for i := 0; i < length; i++ {
		index := rand.Intn(len(sourceStr))
		passwd[i] = sourceStr[index]
	}
	return string(passwd)
}
